<#ibiztemplate>
TARGET=PSSYSAPP
</#ibiztemplate>
<#if sysrun.getPSDBDevInst()??>
    <#assign dbinst = sysrun.getPSDBDevInst()>
    <#assign curdate=.now?string["yyyyMMddHHmmss"]>
</#if>
<?xml version="1.1" encoding="UTF-8" standalone="no"?>
<databaseChangeLog xmlns="http://www.liquibase.org/xml/ns/dbchangelog" xmlns:ext="http://www.liquibase.org/xml/ns/dbchangelog-ext" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.liquibase.org/xml/ns/dbchangelog-ext http://www.liquibase.org/xml/ns/dbchangelog/dbchangelog-ext.xsd http://www.liquibase.org/xml/ns/dbchangelog http://www.liquibase.org/xml/ns/dbchangelog/dbchangelog-3.6.xsd">
    <#assign sequenceNumber=0>

<#list app.getPSSystem().getAllPSDataEntities() as de>
<#if de.getTableName?? &&de.getTableName()??  && de.isSubSysDE()==false>
    <!--输出实体[${de.getName()}]数据结构 -->
    <#assign sequenceNumber=sequenceNumber+1>
    <#assign deVersion=de.getVersion()?c>
    <#assign deName=de.getName()?lower_case>
    <#comment>输出表结构</#comment>
    <#if !P.exists("table_",de.getTableName(),"")>
    <changeSet author="${dbinst.getUserName()}" id="tab-${deName}-${deVersion}-${sequenceNumber}">
        <createTable tableName="${de.getTableName()}">
         <#list de.getAllPSDEFields() as defield>
             <#if defield.isPhisicalDEField()==true><#comment>物理属性</#comment>
                 <#assign dataType="${srfdatatype(defield.getStdDataType())}">
                 <#comment>varchar需要设置字段长度，若不设置，则liquibase比较时会自动填充，最终恢复到生产库会报错</#comment>
                 <#if srfjavatype(defield.stdDataType)=='String'>
                     <#if defield.getLength()?? && defield.getLength()?c!='-1'>
                         <#assign dataType="${srfdatatype(defield.getStdDataType())}(${defield.getLength()?c})" >
                     <#else>
                         <#assign dataType="${srfdatatype(defield.getStdDataType())}(200)">
                     </#if>
                 <#elseif dataType?lower_case=='decimal'>
                         <#assign dataType="${srfdatatype(defield.getStdDataType())}(38,2)"><#comment>设置数值类型精度</#comment>
                 </#if>
                 <#comment>由于liquibase不支持修改mysql的remarks，修改remarks会导致启动报错，所以mysql数据库暂时不发remarks</#comment>
                <column name="${defield.getName()}" remarks="<#if (dbinst.getDBType()!='MYSQL5')>${defield.getLogicName()}</#if>" type="${dataType}">
                <#if defield.isKeyDEField()==true>
                    <#comment>oracle中约束名长度不能大于30</#comment>
                    <#assign constraintName="PK_"+de.getName()+"_"+defield.getName()>
                    <#if constraintName?length gt 30>
                        <#assign constraintName=constraintName?substring(0,30)>
                    </#if>
                    <constraints primaryKey="true" primaryKeyName="${constraintName}"/>
                </#if>
                </column>
             </#if>
         </#list>
        </createTable>
    </changeSet>
    </#if>
</#if>
</#list>

  <#comment>输出视图信息</#comment>
<#list app.getPSSystem().getAllPSDataEntities() as de>
    <#if de.getViewName?? && ((de.getViewName()!'')!='') && de.isSubSysDE()==false && de.isNoViewMode()!=true>
    <#assign sequenceNumber=sequenceNumber+1>
    <#assign deVersion=de.getVersion()?c>
    <#assign deName=de.getName()?lower_case>
    <#if !P.exists("view_",de.getViewName(),"")>
    <!--输出实体[${de.getName()}]视图结构信息 -->
     <changeSet author="${dbinst.getUserName()}" id="view-${deName}-${deVersion}-${sequenceNumber}">
         <createView fullDefinition="false" replaceIfExists="true" viewName="${de.getViewName()}">
            <#if de.getAllPSDEDataQueries?? && de.getAllPSDEDataQueries()??>
                <#assign resultDataQuery = de.getAllPSDEDataQueries()>
                <#list resultDataQuery as singleQuery>
                    <#if singleQuery.getCodeName()?lower_case=='default'>
                        <#if singleQuery.getAllPSDEDataQueryCodes()??>
                            <#assign queryCodeList = singleQuery.getAllPSDEDataQueryCodes()>
                            <#assign sql="">
                            <#comment>遍历数据查询，有oracle则输出，无则输出mysql</#comment>
                            <#list singleQuery.getAllPSDEDataQueryCodes() as dedqcode>
                                <#assign sql=srfjavasqlcode('${dedqcode.getQueryCode()}')>
                                <#assign dbtype=dedqcode.getDBType()?lower_case?replace("mysql5","mysql")>
                                <#if dbtype?lower_case=='oracle'>
                                    <#break>
                                </#if>
                            </#list>
               <![CDATA[ ${sql}]]>
                            <#break>
                        </#if>
                    </#if>
                </#list>
            </#if>
         </createView>
     </changeSet>
        </#if>
    </#if>
</#list>

<#comment>输出外键</#comment>
<#list app.getPSSystem().getAllPSDataEntities() as de>
    <#assign deVersion=de.getVersion()?c>
    <#assign deName=de.getName()?lower_case>
    <#if de.getTableName?? &&de.getTableName()??   && de.isSubSysDE()==false>
        <#if de.getMinorPSDERs?? && de.getMinorPSDERs()?? >
        <!--输出实体[${de.getName()}]外键关系 -->
            <#list de.getMinorPSDERs() as fk>
                <#if fk.getDERType()=='DER1N'>
                    <#if fk.getMajorPSDataEntity().getTableName()??>
                        <#assign sequenceNumber=sequenceNumber+1>
                        <#comment>oracle中约束名长度不能大于30</#comment>
                        <#assign fkConstraintName=fk.getName()>
                        <#if fkConstraintName?length gt 30>
                            <#assign fkConstraintName=fkConstraintName?substring(0,30)>
                        </#if>
    <#if !P.exists("fk_",fkConstraintName,"")>
    <changeSet author="${dbinst.getUserName()}" id="fk-${deName}-${deVersion}-${sequenceNumber}">
       <addForeignKeyConstraint baseColumnNames="${fk.getPSPickupDEField().getName()}" baseTableName="${de.getTableName()}" constraintName="${fkConstraintName}" deferrable="false" initiallyDeferred="false" onDelete="RESTRICT" onUpdate="RESTRICT" referencedColumnNames="${fk.getMajorPSDataEntity().getKeyPSDEField().getName()}" referencedTableName="${fk.getMajorPSDataEntity().getTableName()}" validate="true"/>
    </changeSet>
    </#if>
                    </#if>
                </#if>
            </#list>
        </#if>
    </#if>
</#list>

</databaseChangeLog>
